# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from .logger import function_log, logger

# TODO add auto settings for cloud platforms

__all__ = [
    'function_log',
    'logger'
]
